#include "config.h"
#include "string.h"
#include "ai_usb.h"

void initKeyboard();
void keyboardRun(u8);

extern BYTE bKeyCode;
extern BYTE bKeyDebounce;
extern BOOL fKeyOK;

BOOL keyboardInBusy;
BYTE lastKeyCode;

sbit KEY32 = P3^2;
sbit KEY33 = P3^3;
sbit LED_NUM        = P2^0;
sbit LED_CAPS       = P2^1;
sbit LED_SCROLL     = P2^2;

#define Keyboard_InterfaceNum 4
#define Keyboard_EP 4
#define Keyboard_OUT_SIZE 1
BYTE keyboardOutBuffer[Keyboard_OUT_SIZE];

char code KEYBOARD_DESC[32] ={
		0x09,                   //bLength(9);
    0x04,                   //bDescriptorType(Interface);
    Keyboard_InterfaceNum,                   //bInterfaceNumber(2);
    0x00,                   //bAlternateSetting(0);
    0x02,                   //bNumEndpoints(2);
    0x03,                   //bInterfaceClass(HID);
    0x01,                   //bInterfaceSubClass(Boot);
    0x01,                   //bInterfaceProtocol(Keyboard);
    0x00,                   //iInterface(0);

    0x09,                   //bLength(9);
    0x21,                   //bDescriptorType(HID);
    0x01,0x01,              //bcdHID(1.01);
    0x00,                   //bCountryCode(0);
    0x01,                   //bNumDescriptors(1);
    0x22,                   //bDescriptorType(HID Report);
    0x41,0x00,              //wDescriptorLength(65);

    0x07,                   //bLength(7);
    0x05,                   //bDescriptorType(Endpoint);
    0x80 | Keyboard_EP,                   //bEndpointAddress(EndPoint2 as IN);
    0x03,                   //bmAttributes(Interrupt);
    0x08,0x00,              //wMaxPacketSize(8);
    0x0a,                   //bInterval(10ms);

    0x07,                   //bLength(7);
    0x05,                   //bDescriptorType(Endpoint);
    Keyboard_EP,                   //bEndpointAddress(EndPoint2 as OUT);
    0x03,                   //bmAttributes(Interrupt);
    Keyboard_OUT_SIZE,0x00,              //wMaxPacketSize(1);
    0x0a,                   //bInterval(10ms);
};


/*
Input Report:
0   Modifierkeys (D0:LCtrl D1:LShift D2:LAlt D3:LGui D4:RCtrl D5:RShift D6:RAlt D7:RGui)
1	Reserved
2	Keycode 1
3	Keycode 2
4	Keycode 3
5	Keycode 4
6	Keycode 5
7	Keycode 6
Output Report:
0   LEDs (D0:NumLock D1:CapLock D2:ScrollLock)
*/
char code KEYBOARDREPORTDESC[65] =
{
    0x05,0x01,              //USAGE_PAGE(Generic Desktop);
    0x09,0x06,              //USAGE(Keyboard);
    0xa1,0x01,              //COLLECTION(Application);
    0x05,0x07,              //  USAGE_PAGE(Keyboard);
    0x19,0xe0,              //  USAGE_MINIMUM(224);
    0x29,0xe7,              //  USAGE_MAXIMUM(255);
    0x15,0x00,              //  LOGICAL_MINIMUM(0);
    0x25,0x01,              //  LOGICAL_MAXIMUM(1);
    0x75,0x01,              //  REPORT_SIZE(1);
    0x95,0x08,              //  REPORT_COUNT(8);
    0x81,0x02,              //  INPUT(Data,Variable,Absolute);
    0x75,0x08,              //  REPORT_SIZE(8);
    0x95,0x01,              //  REPORT_COUNT(1);
    0x81,0x01,              //  INPUT(Constant);
    0x19,0x00,              //  USAGE_MINIMUM(0);
    0x29,0x65,              //  USAGE_MAXIMUM(101);
    0x15,0x00,              //  LOGICAL_MINIMUM(0);
    0x25,0x65,              //  LOGICAL_MAXIMUM(101);
    0x75,0x08,              //  REPORT_SIZE(8);
    0x95,0x06,              //  REPORT_COUNT(6);
    0x81,0x00,              //  INPUT(Data,Array);
    0x05,0x08,              //  USAGE_PAGE(LEDs);
    0x19,0x01,              //  USAGE_MINIMUM(1);
    0x29,0x03,              //  USAGE_MAXIMUM(3);
    0x15,0x00,              //  LOGICAL_MINIMUM(0);
    0x25,0x01,              //  LOGICAL_MAXIMUM(1);
    0x75,0x01,              //  REPORT_SIZE(1);
    0x95,0x03,              //  REPORT_COUNT(3);
    0x91,0x02,              //  OUTPUT(Data,Variable,Absolute);
    0x75,0x05,              //  REPORT_SIZE(5);
    0x95,0x01,              //  REPORT_COUNT(1);
    0x91,0x01,              //  OUTPUT(Constant);
    0xc0,                   //END_COLLECTION;
};


void keyboardInFun(){
	  BYTE csr;

    usb_write_reg(INDEX, Keyboard_EP);
    csr = usb_read_reg(INCSR1);
    if (csr & INSTSTL)
    {
        usb_write_reg(INCSR1, INCLRDT);
    }
    if (csr & INUNDRUN)
    {
        usb_write_reg(INCSR1, 0);
    }

    keyboardInBusy = 0;
};
void _keyboardOutFun()
{
    BYTE led;
    if (usb_bulk_intr_out(keyboardOutBuffer, Keyboard_EP) == 1)
    {
        led = keyboardOutBuffer[0];
        LED_NUM = !(led & 0x01);
        LED_CAPS = !(led & 0x02);
        LED_SCROLL = !(led & 0x04);
    }
}
void keyboardOutFun(){
	    BYTE csr;

    usb_write_reg(INDEX, Keyboard_EP);
    csr = usb_read_reg(OUTCSR1);
    if (csr & OUTSTSTL)
    {
        usb_write_reg(OUTCSR1, OUTCLRDT);
    }
    if (csr & OUTOPRDY)
    {
        _keyboardOutFun();
    }
};

bit isAltHold;
void initKeyboard(){
		P3M0 = 0x00; P3M1 = 0x00; 
		lastKeyCode = 0xFF;
		bKeyDebounce = 0;
		keyboardInBusy = 0;
		set_usb_ep_callback(Keyboard_EP,64,keyboardInFun,64,keyboardOutFun);
		usb_add_interface(Keyboard_InterfaceNum,KEYBOARD_DESC,sizeof(KEYBOARD_DESC));
		usb_add_hidDesc(Keyboard_InterfaceNum,KEYBOARDREPORTDESC,sizeof(KEYBOARDREPORTDESC));
		isAltHold = 0;
};

void keyboardRun(u8 inputCode){
    BYTE key[8];
		u8 i;
		BYTE tempCode = bKeyCode & 0x03;
		if (!keyboardInBusy && (inputCode != lastKeyCode )){
				lastKeyCode = inputCode;
				key[0] = 0;
				key[1] = 0;
				key[2] = 0;
				key[3] = 0;
				key[4] = 0;
				key[5] = 0;
				key[6] = 0;
				key[7] = 0;
				if(inputCode == 0xFF){
					key[2] = 0;
				}else{
					if(inputCode == 0){
						key[2] = 0x27;
					}else if(inputCode <=9){
						key[2] = 0x1e + inputCode - 1;
					}else if(inputCode >9 && inputCode<16){
						switch(inputCode){
							case 10://A  Ctrl+N
								key[0] = 0x01;	
								key[2] = 0x11;
								break;
							case 11://B  Alt+N
								key[0] = 0x04;	
								key[2] = 0x11;
								break;
							case 12://C  Alt+Tab
								key[0] = 0x04;	
								key[2] = 0x2B;
								break;
							case 13://D  Alt Hold
								isAltHold  = !isAltHold ;
								key[2] = 0;
								break;
							case 14://E  Tab
								key[2] = 0x2B;
								break;
							case 15://F  SPACE
								key[2] = 0x2C;
								break;
							default:
								//key[2] = 0x04 + inputCode - 10;
								break;
						};
						
					}else {
						key[2] = 0;
					}
					
				}
				if(isAltHold){
					key[0] = 0x04;	
				}
				
				//IE2 &= ~EUSB;
				EUSB = 0;
				keyboardInBusy = 1;
				usb_write_reg(INDEX, Keyboard_EP);
				for (i=0; i<8; i++)
				{
						usb_write_reg(FIFO0+Keyboard_EP, key[i]);
				}
				usb_write_reg(INCSR1, INIPRDY);
				//IE2 |= EUSB;
				EUSB = 1;
		}
}