import os
import ctypes
import usb.core
import usb.util
import usb.backend.libusb1 as libusb1
import usb.backend.libusb0 as libusb0
import array

# import usb.backend.openusb as openusb
# 获取当前脚本的绝对路径
script_dir = os.path.dirname(os.path.abspath(__file__))
#os.environ["PYUSB_DEBUG"] = "debug"
#os.environ["LIBUSB_DEBUG"] = "4"

# 切换到脚本所在目录
os.chdir(script_dir)
os.add_dll_directory(os.getcwd())  # 添加当前目录
os.environ['PATH'] = os.getcwd()+";"+os.environ['PATH']

def myFindLibrary(name):
    if name == "usb-1.0":
        return  os.path.join(os.getcwd(),".\libusb-1.0.dll") 
    if name == "usb0":
        return  os.path.join(os.getcwd(),".\libusb0.dll") 
    return ctypes.util.find_library(name)
    
m=libusb1
#m=libusb0
backend = m.get_backend(myFindLibrary)

if backend is not None:
    print(f'find(): using backend "{m.__name__}"')
            
# 定义 USB 设备的厂商 ID 和产品 ID，需要替换为你实际设备的值
VENDOR_ID = 0x34BF
#PRODUCT_ID = 0xFF01

#usb.core.
# for printer in usb.core.find(find_all=True,backend=backend):
#     print (printer)
# 查找 USB 设备
#, idProduct=PRODUCT_ID
device = usb.core.find(idVendor=VENDOR_ID,backend=backend)

if device is None:
    print("未找到指定的 USB 设备。")
else:
    try:
        # 如果设备被内核驱动占用，需要先分离内核驱动
        # if device.is_kernel_driver_active(0):
        #     device.detach_kernel_driver(0)
        #     print("已分离内核驱动。")

        # 声明接口
        configIndex = 2;
        device.set_configuration()
        cfg = device.get_active_configuration()
        interface_number = cfg[(configIndex, 0)].bInterfaceNumber
        for i in cfg:
            print(i)
    
        #usb.util.claim_interface(device, 0)
        #usb.util.claim_interface(device, 1)
        usb.util.claim_interface(device, interface_number)
        print("已声明接口。")

        # 查找批量输入和输出端点
        endpoints = usb.util.find_descriptor(cfg[(configIndex, 0)], find_all=True, custom_match=lambda e: (
                usb.util.endpoint_direction(e.bEndpointAddress) in (usb.util.ENDPOINT_IN, usb.util.ENDPOINT_OUT)
        ))
        in_endpoint = None
        out_endpoint = None
        for endpoint in endpoints:
            if usb.util.endpoint_direction(endpoint.bEndpointAddress) == usb.util.ENDPOINT_IN:
                in_endpoint = endpoint
            elif usb.util.endpoint_direction(endpoint.bEndpointAddress) == usb.util.ENDPOINT_OUT:
                out_endpoint = endpoint

        if in_endpoint is None or out_endpoint is None:
            print("未找到批量输入或输出端点。")
        else:
            # 准备要发送的数据
            data_to_send = b"Hello, USB Device!"
            random_data = os.urandom(248)  # 生成 64 字节的随机字节串
            print(f"Sending data: {random_data.hex()}")  # 以十六进制格式打印数据
            #out_endpoint.write(data_to_send)
            # 发送数据到设备
            bytes_sent = device.write(out_endpoint.bEndpointAddress, random_data,timeout=2000)
            print(f"已发送 {bytes_sent} 字节数据: {random_data.hex()}")
            # 从设备接收数据
            received_data = device.read(in_endpoint.bEndpointAddress, in_endpoint.wMaxPacketSize,timeout=1000)
            # 如果 received_data 是 array.array 对象，将其转换为 bytes 对象
            if isinstance(received_data, array.array):
                received_data = bytes(received_data)

            received_hex = received_data.hex()
            print(f"接收到 {len(received_data)} 字节数据: {received_hex}")
            if received_hex != random_data.hex():
                print("接收到的数据与预期不符.")
                raise Exception("接收到的数据与预期不符.")
            else:
                print("接收到的数据与预期一致.")
    except usb.core.USBError as e:
        print(f"USB 通信错误: {e}")
    except Exception as e:
        print(f"USB 错误: {e}")
    finally:
        # 释放接口并重新附加内核驱动（如果之前分离了）
        # if device is not None:
        #     usb.util.release_interface(device, interface_number)
        #     if device.is_kernel_driver_active(0):
        #         device.attach_kernel_driver(0)
        #     print("已释放接口并重新附加内核驱动。")
        pass
print("程序结束")
