import numpy as np
import pyautogui
import cv2
import serial
import time
from PIL import Image, ImageOps
import threading
from queue import Queue
import copy

def capture_desktop():
    # 抓取桌面图片
    screenshot = pyautogui.screenshot()
    return screenshot

def _convert_to_rgb565(image):
    # 等比缩放图片为320x240像素
    image = image.resize((320, 240), Image.Resampling.LANCZOS)
    image = image.rotate(180)
    # 图片左右镜像翻转
    image = ImageOps.mirror(image)
    # 将图片转换为RGB模式
    image = image.convert("RGB")

    width, height = image.size
    rgb565_data = bytearray()
    for x in range(width):
        for y in range(height):
            r, g, b = image.getpixel((x, y))
            # 将RGB888转换为RGB565
            r = (r >> 3) & 0x1F
            g = (g >> 2) & 0x3F
            b = (b >> 3) & 0x1F

            rgb565 = (r << 11) | (g << 5) | b
            # 将16位的RGB565拆分为两个8位字节
            rgb565_data.extend([(rgb565 >> 8) & 0xFF, rgb565 & 0xFF])
    with open('src.rgb565', 'wb') as f:
        f.write(rgb565_data)
    return rgb565_data

def convert_to_rgb565(image):
    #_convert_to_rgb565(copy.deepcopy(image));
    # 等比缩放图片为 320x240 像素
    image = image.resize((320, 240), Image.Resampling.LANCZOS)
    image = image.rotate(180)
    # 图片左右镜像翻转
    image = ImageOps.mirror(image)
    # 将图片转换为 RGB 模式
    image = image.convert("RGB")

    img_array = np.array(image, dtype=np.int32)
    r = img_array[:, :, 0]
    g = img_array[:, :, 1]
    b = img_array[:, :, 2]

    # 将 RGB888 转换为 RGB565
    r = (r >> 3) & 0x1F
    g = (g >> 2) & 0x3F
    b = (b >> 3) & 0x1F

    # 合并 RGB565 数据
    rgb565 = (r << 11) | (g << 5) | b

    # 按列优先处理
    rgb565 = rgb565.T.flatten()

    # 拆分 16 位的 RGB565 为两个 8 位字节
    high_byte = (rgb565 >> 8).astype(np.uint8)
    low_byte = (rgb565 & 0xFF).astype(np.uint8)

    # 正确合并高低字节
    rgb565_bytes = np.zeros(high_byte.size * 2, dtype=np.uint8)
    rgb565_bytes[::2] = high_byte
    rgb565_bytes[1::2] = low_byte
    # with open('op.rgb565', 'wb') as f:
    #     f.write(bytearray(rgb565_bytes))
    return bytearray(rgb565_bytes)

def send_data(ser, data_queue, packet_size=51200):
    n = 0
    timeInterval = 0.005
    start_time = time.time()
    while True:
        n += 1
        data = data_queue.get()
        print(f'send:{len(data)}')
        for i in range(0, len(data), packet_size):
            packet = data[i:i + packet_size]
            ser.write(packet)
            time.sleep(timeInterval)
            # 计算并打印帧率
            current_time = time.time()
            elapsed_time = current_time - start_time
            fps = n / elapsed_time
            print(f"当前帧率: {fps:.2f} FPS")


def capture(data_queue):
    while True:
        a = time.time()
        
        # 抓取桌面图片
        image = capture_desktop()
        

        #a = time.time()
        # 转换为RGB565格式
        data = convert_to_rgb565(image)
        #b = time.time()
        #print(f'time to convert: {b - a:.2f}')
        b = time.time()
        #print(f'time to capture: {b - a:.2f}')
        # 确保队列中只有一个最新的数据
        while not data_queue.empty():
            data_queue.get()

        data_queue.put(data)


def main():
    port = 'COM45'  # 根据实际情况修改串口名称
    baudrate = 5000000  # 根据实际情况修改波特率
    try:
        ser = serial.Serial(port, baudrate)
        rcvData = ser.read()
        print(rcvData.decode())
        if 'r' in rcvData.decode():
            desktop_queue = Queue()
            data_queue = Queue()

            capture_thread = threading.Thread(target=capture, args=(data_queue,))
            send_thread = threading.Thread(target=send_data, args=(ser, data_queue))

            capture_thread.start()
            send_thread.start()

            capture_thread.join()
            send_thread.join()
    except serial.SerialException as e:
        print(f"串口连接错误: {e}")
    except Exception as e:
        print(f"发生未知错误: {e}")
    finally:
        if ser.is_open:
            ser.close()


if __name__ == "__main__":
    main()
    